import factory

from .models import User


class UserFactory(factory.DjangoModelFactory):
    name = factory.Sequence('User {}'.format)
    username = factory.Sequence('username{}'.format)
    email = factory.Sequence('email{}@incuna.com'.format)

    class Meta:
        model = User



import os
import sys

import dj_database_url
import django
from colour_runner.django_runner import ColourRunnerMixin
from django.conf import settings
from django.test.runner import DiscoverRunner


BASEDIR = os.path.dirname(os.path.dirname(__file__))

settings.configure(
    DATABASES={
        'default': dj_database_url.config(
            default='sqlite://{}/user_deletion.db'.format(BASEDIR),
        ),
    },
    INSTALLED_APPS=(
        'tests',
        'user_deletion',

        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sites',
    ),
    MIDDLEWARE_CLASSES=(),

    AUTH_USER_MODEL='tests.User',
    SITE_ID=1,
    DEFAULT_FROM_EMAIL='from@example.com',
)


django.setup()


class TestRunner(ColourRunnerMixin, DiscoverRunner):
    """Enable colorised output."""


test_runner = TestRunner(verbosity=1)
failures = test_runner.run_tests(['tests'])
if failures:
    sys.exit(1)



from dateutil.relativedelta import relativedelta
from django.apps import apps
from django.utils import timezone


class UserDeletionManagerMixin:
    def users_to_notify(self):
        """Finds all users who have been inactive and not yet notified."""
        user_deletion_config = apps.get_app_config('user_deletion')

        threshold = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_NOTIFICATION,
        )
        return self.filter(last_login__lte=threshold, notified=False)

    def users_to_delete(self):
        """Finds all users who have been inactive and were notified."""
        user_deletion_config = apps.get_app_config('user_deletion')

        threshold = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_DELETION,
        )
        return self.filter(last_login__lte=threshold, notified=True)



from dateutil.relativedelta import relativedelta
from django.apps import apps
from django.test import TestCase
from django.utils import timezone

from .factories import UserFactory
from .models import User


user_deletion_config = apps.get_app_config('user_deletion')


class TestUserDeletionManager(TestCase):
    def test_users_to_notify(self):
        last_login = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_NOTIFICATION,
        )
        user = UserFactory.create(last_login=last_login, notified=False)
        users = User.objects.users_to_notify()

        self.assertCountEqual(users, [user])

    def test_users_not_to_notify(self):
        user = UserFactory.create(last_login=timezone.now(), notified=False)
        users = User.objects.users_to_notify()

        self.assertNotIn(user, users)

    def test_users_already_notified(self):
        last_login = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_NOTIFICATION,
        )
        user = UserFactory.create(last_login=last_login, notified=True)
        users = User.objects.users_to_notify()

        self.assertNotIn(user, users)

    def test_users_to_delete(self):
        last_login = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_DELETION,
        )
        user = UserFactory.create(last_login=last_login, notified=True)
        users = User.objects.users_to_delete()

        self.assertCountEqual(users, [user])

    def test_users_not_to_delete(self):
        user = UserFactory.create(last_login=timezone.now(), notified=False)
        users = User.objects.users_to_delete()

        self.assertNotIn(user, users)

    def test_users_to_delete_not_notified(self):
        last_login = timezone.now() - relativedelta(
            months=user_deletion_config.MONTH_DELETION,
        )
        user = UserFactory.create(last_login=last_login, notified=False)
        users = User.objects.users_to_delete()

        self.assertNotIn(user, users)



from unittest.mock import patch

from dateutil.relativedelta import relativedelta
from django.core import mail
from django.core.management import call_command
from django.test import TestCase
from django.utils import timezone

from .factories import UserFactory
from .models import User


MONTH = 1


@patch('user_deletion.apps.UserDeletionConfig.MONTH_NOTIFICATION', new=MONTH)
class TestUserNotifyManagementCommand(TestCase):
    def test_no_users(self):
        call_command('notify_users')

        self.assertEqual(len(mail.outbox), 0)

    def test_inactive_users(self):
        month_ago = timezone.now() - relativedelta(months=MONTH)
        user = UserFactory.create(last_login=month_ago, notified=False)

        call_command('notify_users')

        self.assertEqual(len(mail.outbox), 1)
        user.refresh_from_db()
        self.assertTrue(user.notified)

    def test_notified_users(self):
        year_ago = timezone.now() - relativedelta(months=MONTH)
        UserFactory.create(last_login=year_ago, notified=True)

        call_command('notify_users')

        self.assertEqual(len(mail.outbox), 0)


@patch('user_deletion.apps.UserDeletionConfig.MONTH_DELETION', new=MONTH)
class TestDeleteUsersManagementCommand(TestCase):
    def test_no_users(self):
        call_command('delete_users')
        self.assertEqual(len(mail.outbox), 0)

    def test_inactive_users(self):
        year_ago = timezone.now() - relativedelta(months=MONTH)
        user = UserFactory.create(last_login=year_ago, notified=True)

        call_command('delete_users')

        self.assertEqual(len(mail.outbox), 1)
        with self.assertRaises(User.DoesNotExist):
            user.refresh_from_db()



from django.contrib.sites.models import Site
from django.core import mail
from django.test import TestCase

from user_deletion.notifications import (
    AccountDeletedNotification,
    AccountInactiveNotification,
)
from .factories import UserFactory


class TestAccountDeletedNotification(TestCase):
    def test_notification(self):
        user = UserFactory.create()
        site = Site.objects.get_current()

        AccountDeletedNotification(user=None, site=site, users=[user]).notify()

        email = mail.outbox[0]
        self.assertEqual(email.subject, 'Your account has been deleted')
        self.assertIn('You were informed', email.body)


class TestAccountInactiveNotification(TestCase):
    def test_notification(self):
        user = UserFactory.create()
        site = Site.objects.get_current()

        AccountInactiveNotification(user=None, site=site, users=[user]).notify()

        email = mail.outbox[0]
        self.assertEqual(email.subject, 'Re-activate your account')
        self.assertIn('We have noticed', email.body)



from django.contrib.auth.models import AbstractUser
from django.db import models

from user_deletion.managers import UserDeletionManagerMixin


class UserManager(UserDeletionManagerMixin, models.Manager):
    pass


class User(AbstractUser):
    name = models.CharField(max_length=255)
    notified = models.BooleanField(default=False)

    objects = UserManager()

    def __str__(self):
        return self.name
